#include "ps2kbd.h"
#include "scancodes.h"
//#include "uart.h"
//#include "systime.h"

extern BOOL hold;

//#define bit_test(var) ((var & (1<<bit)) != 0)
uint8_t kbdbits;    // number of received bits (start bit inclusive)
uint8_t kbdbyte;    // data send/recieve
uint8_t kbdto;      // timeout for a keystroke receiving
//BOOL kbdtx;         // 1 when transmitting to keyboard, 0 when receiving
BOOL kbdbit;        // received bit
BOOL kbdoddparity;  // 1 if odd number of bits before parity
BOOL kbderr;        // 1 if any errors
//BOOL acked;        // keyboard acknowledge response
BOOL escaped;      // escaped code 0xE0
unsigned char	keyBuf[KBUFFSIZE];	//PS2 Keyboard buffer, the register to store characters key in
uint8_t	keyIn;			//Index into PS/2 key buf where next scan code will be inserted
uint8_t	keyOut;			//Index into PS/2 key buf where next scan code will be removed
uint8_t	keyRead;		//Number of keys read from the PS/2 keyboard

DWORD cfgKeyboard(void)
{
    KClock_Clr(); //hold keyboard clock/data line untill ready
    KData_Clr();
    EXTMODE = 0x00000002;	/* INT1 edge trigger */
    EXTPOLAR = 0;			/* INT1 is falling edge by default */

    PINSEL0 = (PINSEL0 & ~KBD_PINMASK) | KBD_PINSEL;
    /* initialize the interrupt vector */
    VICIntSelect &= ~VIC_BIT(VIC_EINT1);  // EINT1 selected as IRQ
    VICVectAddr2 = (uint32_t)extISR1;     // address of the ISR
    VICVectCntl2 = VIC_ENABLE | VIC_EINT1;
    VICIntEnable = VIC_BIT(VIC_EINT1);    // EINT1 interrupt enabled

    keyIn =0;
    keyOut=0;
    keyRead=0;

    kbdbits=0;          // number of received bits (start bit inclusive)
    kbdbyte=0;          // receiving byte
    kbdto=0;            // timeout for a keystroke receiving
//    kbdtx=0;        // 1 when transmitting to keyboard, 0 when receiving
    kbdbit=0;           // received bit
    kbdoddparity=0;     // 1 if odd number of bits before parity
    kbderr=0;           // 1 if any errors

    EXTINT = 0x00000002;	/* clear interrupt */
    /* Release clock/data line */
    KClock_Set();
    KData_Set();
    return(TRUE);
}

void decode(unsigned char sc)
{
    static unsigned char is_up=0; /*shift_state = 0,*/ //, mode = 0;
    unsigned char i;
    if (!is_up)            /* Last data received was the up-key identifier */
    {
        switch (sc)
        {
        case 0xF0 :        /*  The up-key identifier */
            is_up = 1;
            break;
        case 0xE0 :
            escaped = 1;
            break;
        case 0x12 :
        case 0x59:
            kb_flags.shift = 1; //!is_up;
            break;
        case 0x14:  //LEFT CONTROL
//            kb_flags.ctrl = !is_up;
            break;
        case 0x11:  //LEFT ALT
//            kb_flags.alt = !is_up;
            break;
        case 0x58:  //CAPS LOCK
            kb_flags.capsLock^=1;
//            SetKbd();
            break;
        case 0x77:  // NUM LOCK
            kb_flags.numLock^=1;  //Num lock always active!
//            SetKbd();
            break;
        case 0x7E:  // SCROLL LOCK
            break;
        case 0xAA:  // Self Test
            break;
        case 0xFA:  // ACK
//            acked = TRUE;
            break;
        case K_F12:
            hold ^= 1;
            break;
        default:
            if (escaped)
            {
                for (i=0; extended[i][0]!=sc && extended[i][0]; i++);
                if (extended[i][0] == sc)
                    putkeybuf((extended[i][1]));
                escaped = 0;
            }
            else
            {
                if (kb_flags.shift^kb_flags.capsLock)          /* If shift pressed */
                {
                    /* look-up */
                    for (i = 0; shifted[i][0]!=sc && shifted[i][0]; i++);
                    if (shifted[i][0] == sc)
                    {
                        putkeybuf((shifted[i][1]));
                    }
                }
                else
                {
                    // do a table look-up
                    for (i = 0; unshifted[i][0]!=sc && unshifted[i][0]; i++);
                    if (unshifted[i][0] == sc)
                    {
                        putkeybuf((unshifted[i][1]));
                    }
                }

            }
            break;
        }
    }
    else
    {
        is_up = 0;
        switch (sc)
        {
        case LEFTSHIFT :
        case RIGHTSHIFT:
            kb_flags.shift = 0;
            break;
        }
    }
}//decode


/*
**  putkeybuf() - Put character into buffer
**  Parameters:  - Char to put in the buffer
*/
void putkeybuf (unsigned char key)
{
    if (keyRead < KBUFFSIZE)  		//make sure that we don't overflow the buffer
    {
        keyRead++;							//Increment the number of keys read
        keyBuf[keyIn++] = key;		//Store the scan code into the buffer
        if (keyIn >= KBUFFSIZE)  	//Adjust index to the next scan code to put in buffer
        {
            keyIn = 0;
        }
    }
}

/*
**  keyhit()  - Check for keypress
**  Paramters:- None
**  Returns:  - TRUE if a key has been pressed
**			  FALSE if no key pressed
*/
uint8_t keyhit(void)
{
    unsigned char hit;
    hit = (unsigned char)(keyRead > 0)? TRUE : FALSE;
    return(hit);
}
/*
**  xgetkey () - Get keyboard scancode
**  Parameters: - None
**  Returns: !=0xFF is the key scan code of the PS/2 keyboard key pressed
**		   ==0xFF atleast one key is available in the key buffer
*/
unsigned char xgetkey (void)
{
    unsigned char key=0;
    if (keyRead > 0)
    {
        keyRead--;			/* Decrement the number of keys in the buffer */
        key = keyBuf[keyOut];	/* Get scan code from the buffer */
        keyOut++;
        if (keyOut >= KBUFFSIZE)
        {
            keyOut = 0;
        }

    }// else {
    //	return (0xFF);					/* No scan codes in buffer, return -1 */
    //}
    return (key);						/* return the scan code of the key pressed */
}

//static void delay(uint32_t num)
//{
//// 80 micro sec for 1000
//    while (num)
//    {
//        num--;
//    }
//}
//
//void SendByte(uint8_t data)
//{
//    uint8_t i;
//    uint32_t j;
//    for (i=5;i;i--)
//    {
//        uart0Puts("Sending...\r");
//        kbdtx=1;            // tell interrupt routine to tx data
//        kbdbyte=data;
//        kbdbits=0;
//        kbdoddparity=0;
//        kbderr=0;
//        kbdto=20;           // keyboard must complete operation within 15ms
////        for (j=0;j<100;j++); // wait for 100uS
////        kbddata=0;          // pull data low, then release clock
////        kbddatam=0;
//        KClock_Clr();        // pull clock low
////        for (j=0;j<100;j++); // wait for 100uS
//        delay(1500);
//        KData_Clr();        // pull data low,
//        delay(100);
//        KClock_Set();// then release clock
////        while(KClockIn());  // Wait for the kb to bring the Clock line low
//        // ok, start bit was transmitted, then interrupt routine will do the rest
//
//        // wait until data will be TXed
//        for (j=100;j;j--)
//        {
//            pause(TWO_MS);
//            if (kbdtx==0)
//                break;
//        }
////        kbdNewChar();
//        if (j==0)
//        {
//            kbdtx=0;
//            KData_Set();
////            kbddata=1;  // keyboard not connected: set kbd_data as input, then exit
//            return;
//        }
//        if (kbderr)
//        {
//#ifdef DEBUGKBD
//            uart0Puts("kbd_sendData: Data TXed, but got NACK\n");
//#endif
//            pause(TEN_MS); // data transmitted but not acknowledged
//            continue;
//        }
//        for (j=25;j;j--)
//        {
//            // wait for ACK
//            pause(TWO_MS);
//            if (acked)
//            {
//                uart0Puts("Command ACK\r");
//                acked = FALSE;
//                break;
//            }
//            else
//                uart0Puts("Response not ACK\r");
//
////            while (getKeyStroke(&ch),ch)
////            {
////                kbdNewChar();
////                if (ch==0xfa)
////                    break;
////            }
////            if (ch==0xfa)
////            {
////                return;
////            }
////            else if (ch)
////            {
////#ifdef DEBUGKBD
////                rs232puts("kbd_sendData: got answer 0x");
////                rs232putbyte2hex(ch,'\n');
////#endif
////            }
//        }
//    }
//
//
//
//}
//
//void SetKbd(void)
//{
//    // reset keyboard rate & leds accordingly to kbd_table structure
//    // Must be called after keyboard powerOn (when keyboard send 0xAA data)
//    pause(TWENTY_MS);
//    // Don't alter rate and delay... default is 10.9cps and 500ms
//    //kbd_sendData(KBD_CMD_SET_RATE);
//    //kbd_sendData(0x00); // maximum rate, minimum delay
//    SendByte(0xED);
//    SendByte(kb_flags.scrollLock|(kb_flags.numLock<<1)|(kb_flags.capsLock<<2));
////    kbdNewChar();
////    uniKey=0;
//}
//        case 0x05:break; // F1
//        case 0x06:break; // F2
//        case 0x04:break; // F3
//        case 0x0C:break; // F4
//        case 0x03:break; // F5
//        case 0x0B:break; // F6
//        case 0x83:break; // F7
//        case 0x0A:break; // F8
//        case 0x01:break; // F9
//        case 0x09:break; // F10
//        case 0x78:break; // F11
//        case 0x07:break; // F12


